/*
    Copyright (c) 2009 digitalSTROM.org, Zurich, Switzerland

    This file was originally part of the dsplwriter project (GPLv3).

    Adaptions to dss11-ledd:
    Copyright (c) 2012 aizo ag, Zurich, Switzerland


    Author: Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>

    dss11-ledd free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    dss11-ledd is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with dss11-ledd. If not, see <http://www.gnu.org/licenses/>.

*/

#include "config.h"

#include <unistd.h>
#include <stdio.h>
#include <errno.h>
#include <sys/select.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <string.h>
#include <unistd.h>

#include "sockutil.h"

ssize_t sockread(int sockfd, unsigned char *data, size_t len, int timeout)
{
    int ret;
    struct timeval tv;
    fd_set rfds;
    unsigned char *p = data;
    ssize_t bytes_total = 0;
    ssize_t bytes_read = 0;
    int eod = 0;

    while (1)
    {
        FD_ZERO(&rfds);
        FD_SET(sockfd, &rfds);

        if (timeout != 0)
        {
            if (!eod)
            {
                tv.tv_sec = timeout;
                tv.tv_usec = 0;
            }
            else
            {
                tv.tv_sec = 0;
                tv.tv_usec = 0;
            }
            ret = select(sockfd + 1, &rfds, NULL, NULL, &tv);
        }
        else
            ret = select(sockfd + 1, &rfds, NULL, NULL, NULL);

        if (ret == -1)
        {
            if (errno == EINTR)
                continue;

            fprintf(stderr, "Failed to read from socket: %s\n",
                    strerror(errno));
            return socket_select_failed;
        }

        if (ret == 0)
        {
            if (eod)
                return bytes_total;
            else
                return socket_operation_timed_out;
        }

        if (FD_ISSET(sockfd, &rfds))
        {
            if (timeout)
                eod = 1;

            bytes_read = recv(sockfd, p, len, MSG_NOSIGNAL);

            if (bytes_read < 0)
                return socket_recv_failed;

            if (bytes_read == 0)
                break;

            bytes_total += bytes_read;
            len -= bytes_read;

            if (len <= 0)
                break;

            p = data + bytes_total;
        }
    }

    if (bytes_total < 0)
        return socket_error;

    return bytes_total;
}

ssize_t sockwrite(int sockfd, unsigned char *data, size_t len, int timeout)
{
    int ret;
    struct timeval tv;
    fd_set wfds;
    unsigned char *p = data;
    ssize_t bytes_total = 0;
    ssize_t bytes_written = 0;

    while (1)
    {
        FD_ZERO(&wfds);
        FD_SET(sockfd, &wfds);

        if (timeout != 0)
        {
            ret = select(sockfd + 1, NULL, &wfds, NULL, &tv);
        }
        else
            ret = select(sockfd + 1, NULL, &wfds, NULL, NULL);

        if (ret == -1)
        {
            if (errno == EINTR)
                continue;

            fprintf(stderr, "Failed to write to socket: %s\n",
                    strerror(errno));
            return socket_select_failed;
        }

        if (ret == 0)
        {
            return socket_operation_timed_out;
        }

        if (FD_ISSET(sockfd, &wfds))
        {
            bytes_written = send(sockfd, p, len, MSG_NOSIGNAL);

            if (bytes_written < 0)
                return socket_send_failed;

            if (bytes_written == 0)
                break;

            bytes_total += bytes_written;
            len -= bytes_written;

            if (len <= 0)
                break;

            p = data + bytes_total;
        }
    }

    if (bytes_total < 0)
        return socket_error;

    return bytes_total;
}
