/* opkg_message.h - the opkg package management system

   Copyright (C) 2009 Ubiq Technologies <graham.gower@gmail.com>
   Copyright (C) 2003 Daniele Nicolodi <daniele@grinta.net>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2, or (at
   your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.
*/

#ifndef _OPKG_MESSAGE_H_
#define _OPKG_MESSAGE_H_

#include <string.h>
#include <errno.h>
#include <stdint.h>

typedef enum {
	ERROR,	/* error conditions */
	NOTICE,	/* normal but significant condition */
	INFO,	/* informational message */
	DEBUG,	/* debug level message */
	DEBUG2,	/* more debug level message */
} message_level_t;

#define ERROR_UNSPECIFIED                (1UL << 0)
#define ERROR_DOWNLOAD_FAILED            (1UL << 1)
#define ERROR_DEPENDENCY_FAILED          (1UL << 2)
#define ERROR_PACKAGE_ALREADY_INSTALLED  (1UL << 3)
#define ERROR_PACKAGE_NOT_AVAILABLE      (1UL << 4)
#define ERROR_PACKAGE_NOT_FOUND          (1UL << 5)
#define ERROR_PACKAGE_NOT_INSTALLED      (1UL << 6)
#define ERROR_GPG_ERROR                  (1UL << 7)
#define ERROR_MD5_ERROR                  (1UL << 8)
#define ERROR_SHA256_ERROR               (1UL << 9)
#define ERROR_POSTINSTALL_SCRIPT         (1UL << 10)
#define ERROR_PRERM_SCRIPT               (1UL << 11)
#define ERROR_PACKAGE_NOT_DOWNGRADED     (1UL << 12)

void free_error_list(void);
void print_error_list(void);
void opkg_message(message_level_t level, const char *fmt, ...)
				__attribute__ ((format (printf, 2, 3)));
void opkg_error_cause(uint32_t error);
uint32_t opkg_get_error_causes(void);

#define opkg_msg(l, fmt, args...) \
	do { \
		if (l == NOTICE) \
			opkg_message(l, fmt, ##args); \
		else \
			opkg_message(l, "%s: "fmt, __FUNCTION__, ##args); \
	} while (0)

#define opkg_perror(l, fmt, args...) \
	opkg_msg(l, fmt": %s.\n", ##args, strerror(errno))

#endif /* _OPKG_MESSAGE_H_ */
